package be.iminds.ilabt.jfed.highlevel.util;

import be.iminds.ilabt.jfed.highlevel.model.SfaModel;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.highlevel.model.Sliver;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.common.GeniUrn;
import be.iminds.ilabt.jfed.util.common.RFC3339Util;
import io.sentry.Sentry;

import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.time.Instant;
import java.util.*;
import java.util.stream.Collectors;

public class SentryUtil {
    /**
     * Sentry util for functionality in highlevel-library
     */
    public static List<Map<String, ?>> createSentrySlices(SfaModel sfaModel) {
        return sfaModel.getSlices().stream().map(SentryUtil::createSentrySlice).collect(Collectors.toList());
    }
    public static Map<String, ?> createSentrySlice(Slice slice) {
        Map<String, Object> res = new HashMap<>();

        res.put("urn", slice.getUrnString());
        res.put("expiration_date", slice.getExpirationDate() == null ? null : RFC3339Util.instantToRFC3339String(slice.getExpirationDate()));

        if (slice.getStatus() != null) {
                res.put("status", slice.getStatus().detailedToString());
        }

        List<Map<String, ?>> sliversRes = new ArrayList<>();
        for (Sliver sliver : slice.getSliversCopy()) {
            Map<String, Object> sliverMap = new HashMap<>();
            List<String> components = sliver.getComponentInfo().stream()
                    .filter(Objects::nonNull)
                    .map(c -> c.getClientId() == null ? c.getComponentId() : c.getClientId())
                    .filter(Objects::nonNull)
                    .collect(Collectors.toList());

            List<Map<String, String>> errorDetails =
                    sliver.getStatus().getAllErrors().stream()
                            .filter(Objects::nonNull)
                            .map(e -> {
                                Map<String, String> r = new HashMap<String, String>();
                                r.put("error", e.getError());
                                r.put("sliver_urn", e.getSliverUrn() == null ? null : e.getSliverUrn().toString());
                                r.put("component_urn", e.getComponentUrn() == null ? null : e.getComponentUrn().toString());
                                r.put("timestamp", RFC3339Util.instantToRFC3339String(Instant.ofEpochMilli(e.getTimestamp())));
                                return r;
                            }
                            )
                            .collect(Collectors.toList());

            String sliverUrn = sliver.getUrn() != null ?
                    sliver.getUrn().toString() :
                    "SLIVER-ON: " + sliver.getServer().getDefaultComponentManagerUrn();

            sliverMap.put("urn", sliverUrn);
            sliverMap.put("status_str", sliver.getStatusString());
            sliverMap.put("status", sliver.getStatus() == null ? null :
                    Objects.toString(sliver.getStatus().getGlobalStatus(), null));
            sliverMap.put("expiration_date", sliver.getExpirationDate() == null ? null : RFC3339Util.instantToRFC3339String(sliver.getExpirationDate()));
            sliverMap.put("manifest_rspec", sliver.getManifestRspec() != null);
            sliverMap.put("components", components);
            sliverMap.put("error_details", errorDetails);
            sliversRes.add(sliverMap);
        }
        res.put("slivers", sliversRes);

        return res;
    }
}
