package be.iminds.ilabt.jfed.preferences;

import be.iminds.ilabt.jfed.git.GitAuthPreferences;
import be.iminds.ilabt.jfed.lowlevel.user.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.lib.RetrySettings;
import be.iminds.ilabt.jfed.lowlevel.ssh_key_info.PuTTYFilesKeyInfo;
import com.google.common.collect.ImmutableMap;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.File;
import java.util.List;

/**
 * User: twalcari
 * Date: 12/15/2015
 * Time: 17:05
 */
public class MockJFedPreferences implements JFedPreferences {

    private static final ImmutableMap<String, Object> config;
    private static final RetrySettings retrySettings = new RetrySettings();

    static {
        config = ImmutableMap.<String, Object>builder()
                .put(CorePreferenceKey.PREF_TESTBEDS_JSON_SOURCE.key, "AUTO")
                .put(CorePreferenceKey.PREF_DEBUG_CPU.key, false)
                .put(CorePreferenceKey.PREF_LOGGING_OVERWRITE.key, false)
                .put(CorePreferenceKey.PREF_LOGGING_ROOT_LEVEL.key, "INFO")
                .put(CorePreferenceKey.PREF_LOGGING_EXTTERM_LEVEL.key, "DEBUG")
                .put(CorePreferenceKey.PREF_TESTBEDSPECIFIC_EXOGENI_FORCE_EXOSM.key, "AUTO")
                .put(CorePreferenceKey.PREF_TESTBEDSPECIFIC_EXOGENI_RECOVER_METHOD.key, "BOTH")
                .build();


    }

    @Override
    public void fillinBlanks() {

    }

    @Override
    public boolean containsPreference(@Nonnull PreferenceKey key) {
        return config.containsKey(key.getKey());
    }

    @Override
    public boolean hasKey(@Nonnull PreferenceKey key) {
        return config.containsKey(key.getKey());
    }

    @Override
    @Nullable
    public String getString(@Nonnull PreferenceKey key) {
        return (String) config.get(key.getKey());
    }

    @Nonnull
    @Override
    public List<String> getStringList(@Nonnull PreferenceKey key) {
        return (List<String>) config.get(key.getKey());
    }

    @Override
    public void setString(@Nonnull PreferenceKey key, @Nonnull String value) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void setStringList(@Nonnull PreferenceKey key, @Nonnull List<String> value) {
        throw new UnsupportedOperationException();
    }

    @Override
    @Nullable
    public Integer getInt(@Nonnull PreferenceKey key) {
        return (Integer) config.get(key.getKey());
    }

    @Override
    public int getInt(@Nonnull PreferenceKey key, int defaultValue) {
        Integer res = getInt(key);
        return res == null ? defaultValue : res;
    }

    @Override
    public void setInt(@Nonnull PreferenceKey key, int value) {

        throw new UnsupportedOperationException();
    }

    @Override
    @Nullable
    public Boolean getBoolean(@Nonnull PreferenceKey key) {
        return (Boolean) config.get(key.getKey());
    }

    @Override
    public boolean getBoolean(@Nonnull PreferenceKey key, boolean defaultValue) {
        Boolean res = getBoolean(key);
        return res == null ? defaultValue : res;
    }

    @Override
    public void setBoolean(@Nonnull PreferenceKey key, boolean value) {
        throw new UnsupportedOperationException();
    }

    @Override
    public File getFile(@Nonnull PreferenceKey key) {
        return (File) config.get(key.getKey());
    }

    @Override
    public void setFile(@Nonnull PreferenceKey key, @Nonnull File value) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void clear(@Nonnull PreferenceKey key) {
        throw new UnsupportedOperationException();
    }

    @Override
    @Nullable
    public File getPropertiesFile() {
        return null;
    }

    @Override
    public void addPreferenceChangeListener(@Nonnull PreferenceKey prefDebugCpu, @Nonnull PreferenceChangeListener preferenceChangeListener) {

    }

    @Override
    public void setRetrySettings(@Nonnull RetrySettings retrySettings) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public RetrySettings getRetrySettings() {
        return retrySettings;
    }

    @Nullable
    @Override
    public GitAuthPreferences getGitAuthPreferences(GeniUser user) {
        return null;
    }

    @Nullable
    @Override
    public PuTTYFilesKeyInfo getCustomSshKeyInfo() {
        return null;
    }
}
